/*
  Java gfx-test framework

   Copyright (C) 2012  Red Hat

This file is part of IcedTea.

IcedTea is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

IcedTea is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with IcedTea; see the file COPYING.  If not, write to the
Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301 USA.

Linking this library statically or dynamically with other modules is
making a combined work based on this library.  Thus, the terms and
conditions of the GNU General Public License cover the whole
combination.

As a special exception, the copyright holders of this library give you
permission to link this library with independent modules to produce an
executable, regardless of the license terms of these independent
modules, and to copy and distribute the resulting executable under
terms of your choice, provided that you also meet, for each linked
independent module, the terms and conditions of the license of that
module.  An independent module is a module which is not derived from
or based on this library.  If you modify this library, you may extend
this exception to your version of the library, but you are not
obligated to do so.  If you do not wish to do so, delete this
exception statement from your version.
 */

package org.gfxtest.testsuites;

import java.awt.Graphics2D;


import org.gfxtest.framework.CommonPathsGenerator;
import org.gfxtest.framework.CommonRenderingStyles;
import org.gfxtest.framework.GfxTest;
import org.gfxtest.framework.TestImage;
import org.gfxtest.framework.TestResult;
import org.gfxtest.framework.annotations.GraphicsPrimitive;
import org.gfxtest.framework.annotations.GraphicsPrimitives;
import org.gfxtest.framework.annotations.TestType;
import org.gfxtest.framework.annotations.TestTypes;
import org.gfxtest.framework.annotations.Transformation;
import org.gfxtest.framework.annotations.Transformations;
import org.gfxtest.framework.annotations.Zoom;



/**
 * Various paths and various rendering styles are used for
 * rendering.
 * 
 * @author Pavel Tisnovsky
 */
@TestType(TestTypes.RENDER_TEST)
@Transformation(Transformations.NONE)
@GraphicsPrimitive(GraphicsPrimitives.PATH)
@Zoom(1)
public class Paths extends GfxTest
{

    /**
     * Draw path consisting of one line only.
     * Line path is constructed using new Path.Float()
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     */
    private static void drawLinePathFloat(TestImage image, Graphics2D graphics2d)
    {
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // draw the line path
        graphics2d.draw(CommonPathsGenerator.createLinePathFloat(image));
    }

    /**
     * Draw path consisting of one line only.
     * Line path is constructed using new Path.Double()
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     */
    private static void drawLinePathDouble(TestImage image, Graphics2D graphics2d)
    {
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // draw the line path
        graphics2d.draw(CommonPathsGenerator.createLinePathDouble(image));
    }

    /**
     * Draw quadratic path.
     * Path is constructed using new Path.Float()
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     */
    private static void drawQuadraticPathFloat(TestImage image, Graphics2D graphics2d)
    {
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // draw the quadratic path
        graphics2d.draw(CommonPathsGenerator.createQuadraticPathFloat(image));
    }

    /**
     * Draw quadratic path.
     * Path is constructed using new Path.Double()
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     */
    private static void drawQuadraticPathDouble(TestImage image, Graphics2D graphics2d)
    {
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // draw the quadratic path
        graphics2d.draw(CommonPathsGenerator.createQuadraticPathDouble(image));
    }

    /**
     * Draw cubic path.
     * Path is constructed using new Path.Float()
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     */
    private static void drawCubicPathFloat(TestImage image, Graphics2D graphics2d)
    {
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // draw the cubic path
        graphics2d.draw(CommonPathsGenerator.createCubicPathFloat(image));
    }

    /**
     * Draw cubic path.
     * Path is constructed using new Path.Double()
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     */
    private static void drawCubicPathDouble(TestImage image, Graphics2D graphics2d)
    {
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // draw the cubic path
        graphics2d.draw(CommonPathsGenerator.createCubicPathDouble(image));
    }

    /**
     * Draw closed path.
     * Path is constructed using new Path.Float()
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     */
    private static void drawClosedPathFloat(TestImage image, Graphics2D graphics2d)
    {
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // draw the closed path
        graphics2d.draw(CommonPathsGenerator.createClosedPathFloat(image));
    }

    /**
     * Draw closed path.
     * Path is constructed using new Path.Double()
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     */
    private static void drawClosedPathDouble(TestImage image, Graphics2D graphics2d)
    {
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // draw the closed path
        graphics2d.draw(CommonPathsGenerator.createClosedPathDouble(image));
    }

    /**
     * Draw crossed path.
     * Path is constructed using new Path.Float()
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     */
    private static void drawCrossedPathFloat(TestImage image, Graphics2D graphics2d)
    {
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // draw the crossed path
        graphics2d.draw(CommonPathsGenerator.createCrossedClosedPathFloat(image));
    }

    /**
     * Draw crossed path.
     * Path is constructed using new Path.Double()
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     */
    private static void drawCrossedPathDouble(TestImage image, Graphics2D graphics2d)
    {
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // draw the crossed path
        graphics2d.draw(CommonPathsGenerator.createCrossedClosedPathDouble(image));
    }

    /**
     * Draw crossed path containing quadratic segment.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     */
    private static void drawCrossedPathFloatContainingQuadraticSegment(TestImage image, Graphics2D graphics2d)
    {
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // draw the crossed path
        graphics2d.draw(CommonPathsGenerator.createCrossedClosedPathContainingQuadraticSegmentFloat(image));
    }

    /**
     * Draw crossed path containing quadratic segment.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     */
    private static void drawCrossedPathDoubleContainingQuadraticSegment(TestImage image, Graphics2D graphics2d)
    {
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // draw the crossed path
        graphics2d.draw(CommonPathsGenerator.createCrossedClosedPathContainingQuadraticSegmentDouble(image));
    }

    /**
     * Draw crossed path containing cubic segment.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     */
    private static void drawCrossedPathFloatContainingCubicSegment(TestImage image, Graphics2D graphics2d)
    {
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // draw the crossed path
        graphics2d.draw(CommonPathsGenerator.createCrossedClosedPathContainingCubicSegmentFloat(image));
    }

    /**
     * Draw crossed path containing cubic segment.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     */
    private static void drawCrossedPathDoubleContainingCubicSegment(TestImage image, Graphics2D graphics2d)
    {
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // draw the crossed path
        graphics2d.draw(CommonPathsGenerator.createCrossedClosedPathContainingCubicSegmentDouble(image));
    }

    /**
     * Path is rendered using stroke paint with default stroke width. Line path
     * is constructed using new Path.Float()
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testLinePathFloatStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // draw the path
        drawLinePathFloat(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Path is rendered using stroke paint with default stroke width.
     * Line path is constructed using new Path.Double()
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testLinePathDoubleStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // draw the path
        drawLinePathDouble(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Path is rendered using stroke paint with default stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testQuadraticPathFloatStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // draw the path
        drawQuadraticPathFloat(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Path is rendered using stroke paint with default stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testQuadraticPathDoubleStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // draw the path
        drawQuadraticPathDouble(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Path is rendered using stroke paint with default stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testCubicPathFloatStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // draw the path
        drawCubicPathFloat(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Path is rendered using stroke paint with default stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testCubicPathDoubleStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // draw the path
        drawCubicPathDouble(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Path is rendered using stroke paint with default stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClosedPathFloatStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // draw the path
        drawClosedPathFloat(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Path is rendered using stroke paint with default stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClosedPathDoubleStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // draw the path
        drawClosedPathDouble(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Path is rendered using stroke paint with default stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testCrossedClosedPathFloatStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // draw the path
        drawCrossedPathFloat(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Path is rendered using stroke paint with default stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testCrossedClosedPathDoubleStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // draw the path
        drawCrossedPathDouble(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Path is rendered using stroke paint with default stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testCrossedPathFloatContainingQuadraticSegmentStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // draw the path
        drawCrossedPathFloatContainingQuadraticSegment(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Path is rendered using stroke paint with default stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testCrossedPathDoubleContainingQuadraticSegmentStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // draw the path
        drawCrossedPathDoubleContainingQuadraticSegment(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Path is rendered using stroke paint with default stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testCrossedPathFloatContainingCubicSegmentStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // draw the path
        drawCrossedPathFloatContainingCubicSegment(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Path is rendered using stroke paint with default stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testCrossedPathDoubleContainingCubicSegmentStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // draw the path
        drawCrossedPathDoubleContainingCubicSegment(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if line path could be clipped by an ellipse shape. Path is
     * rendered using stroke paint with zero stroke width.
     * Line path is constructed using new Path.Float()
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testLinePathFloatZeroStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeZeroThick(graphics2d);
        // draw the path
        drawLinePathFloat(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if line path could be clipped by an ellipse shape. Path is
     * rendered using stroke paint with zero stroke width.
     * Line path is constructed using new Path.Double()
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testLinePathDoubleZeroStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeZeroThick(graphics2d);
        // draw the path
        drawLinePathDouble(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Path is rendered using stroke paint with zero stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testQuadraticPathFloatZeroStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeZeroThick(graphics2d);
        // draw the path
        drawQuadraticPathFloat(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Path is rendered using stroke paint with zero stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testQuadraticPathDoubleZeroStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeZeroThick(graphics2d);
        // draw the path
        drawQuadraticPathDouble(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Path is rendered using stroke paint with zero stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testCubicPathFloatZeroStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeZeroThick(graphics2d);
        // draw the path
        drawCubicPathFloat(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Path is rendered using stroke paint with zero stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testCubicPathDoubleZeroStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeZeroThick(graphics2d);
        // draw the path
        drawCubicPathDouble(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if closed path could be clipped by an ellipse shape. Path is
     * rendered using stroke paint with zero stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClosedPathFloatZeroStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeZeroThick(graphics2d);
        // draw the path
        drawClosedPathFloat(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if closed path could be clipped by an ellipse shape. Path is
     * rendered using stroke paint with zero stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClosedPathDoubleZeroStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeZeroThick(graphics2d);
        // draw the path
        drawClosedPathDouble(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if crossed closed path could be clipped by an ellipse shape. Path is
     * rendered using stroke paint with zero stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testCrossedClosedPathFloatZeroStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeZeroThick(graphics2d);
        // draw the path
        drawCrossedPathFloat(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if crossed closed path could be clipped by an ellipse shape. Path is
     * rendered using stroke paint with zero stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testCrossedClosedPathDoubleZeroStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeZeroThick(graphics2d);
        // draw the path
        drawCrossedPathDouble(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if line path could be clipped by an ellipse shape. Path is rendered
     * using stroke paint with thick stroke width.
     * Line path is constructed using new Path.Float()
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testLineFloatPathThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeThickWidth(graphics2d);
        // draw the path
        drawLinePathFloat(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if line path could be clipped by an ellipse shape. Path is rendered
     * using stroke paint with thick stroke width.
     * Line path is constructed using new Path.Double()
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testLineDoublePathThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeThickWidth(graphics2d);
        // draw the path
        drawLinePathDouble(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if quadratic Bezier path could be clipped by an ellipse shape. Path
     * is rendered using stroke paint with thick stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testQuadraticPathFloatThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeThickWidth(graphics2d);
        // draw the path
        drawQuadraticPathFloat(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if quadratic Bezier path could be clipped by an ellipse shape. Path
     * is rendered using stroke paint with thick stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testQuadraticPathDoubleThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeThickWidth(graphics2d);
        // draw the path
        drawQuadraticPathDouble(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if cubic Bezier path could be clipped by an ellipse shape. Path
     * is rendered using stroke paint with thick stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testCubicPathFloatThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeThickWidth(graphics2d);
        // draw the path
        drawCubicPathFloat(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if cubic Bezier path could be clipped by an ellipse shape. Path
     * is rendered using stroke paint with thick stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testCubicPathDoubleThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeThickWidth(graphics2d);
        // draw the path
        drawCubicPathDouble(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if closed path could be clipped by an ellipse shape. Path
     * is rendered using stroke paint with thick stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClosedPathDoubleThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeThickWidth(graphics2d);
        // draw the path
        drawClosedPathDouble(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if crossed closed path could be clipped by an ellipse shape. Path
     * is rendered using stroke paint with thick stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testCrossedClosedPathFloatThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeThickWidth(graphics2d);
        // draw the path
        drawCrossedPathFloat(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if crossed closed path could be clipped by an ellipse shape. Path
     * is rendered using stroke paint with thick stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testCrossedClosedPathDoubleThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeThickWidth(graphics2d);
        // draw the path
        drawCrossedPathDouble(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if line path could be clipped by an ellipse shape. Path is rendered
     * using stroke paint with extra thick width.
     * Line path is constructed using new Path.Float()
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testLinePathFloatExtraThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeExtraThickWidth(graphics2d);
        // draw the path
        drawLinePathFloat(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if line path could be clipped by an ellipse shape. Path is rendered
     * using stroke paint with extra thick width.
     * Line path is constructed using new Path.Double()
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testLinePathDoubleExtraThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeExtraThickWidth(graphics2d);
        // draw the path
        drawLinePathDouble(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if quadratic Bezier path could be clipped by an ellipse shape. Path
     * is rendered using stroke paint with extra thick width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testQuadraticPathFloatExtraThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeExtraThickWidth(graphics2d);
        // draw the path
        drawQuadraticPathFloat(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if quadratic Bezier path could be clipped by an ellipse shape. Path
     * is rendered using stroke paint with extra thick width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testQuadraticPathDoubleExtraThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeExtraThickWidth(graphics2d);
        // draw the path
        drawQuadraticPathDouble(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if cubic Bezier path could be clipped by an ellipse shape. Path
     * is rendered using stroke paint with extra thick width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testCubicPathFloatExtraThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeExtraThickWidth(graphics2d);
        // draw the path
        drawCubicPathFloat(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if cubic Bezier path could be clipped by an ellipse shape. Path
     * is rendered using stroke paint with extra thick width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testCubicPathDoubleExtraThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeExtraThickWidth(graphics2d);
        // draw the path
        drawCubicPathDouble(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if closed path could be clipped by an ellipse shape. Path
     * is rendered using stroke paint with extra thick width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClosedPathFloatExtraThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeExtraThickWidth(graphics2d);
        // draw the path
        drawClosedPathFloat(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if closed path could be clipped by an ellipse shape. Path
     * is rendered using stroke paint with extra thick width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClosedPathDoubleExtraThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeExtraThickWidth(graphics2d);
        // draw the path
        drawClosedPathDouble(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if crossed closed path could be clipped by an ellipse shape. Path
     * is rendered using stroke paint with extra thick width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testCrossedClosedPathFloatExtraThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeExtraThickWidth(graphics2d);
        // draw the path
        drawCrossedPathFloat(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if crossed closed path could be clipped by an ellipse shape. Path
     * is rendered using stroke paint with extra thick width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testCrossedClosedPathDoubleExtraThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeExtraThickWidth(graphics2d);
        // draw the path
        drawCrossedPathDouble(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if line path could be clipped by an ellipse shape. Path is
     * rendered using stroke paint with default stroke width.
     * Line path is constructed using new Path.Float()
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testLinePathFloatDashedStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d);
        // draw the path
        drawLinePathFloat(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if line path could be clipped by an ellipse shape. Path is
     * rendered using stroke paint with default stroke width.
     * Line path is constructed using new Path.Double()
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testLinePathDoubleDashedStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d);
        // draw the path
        drawLinePathDouble(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if cubic path could be clipped by an ellipse shape. Path is
     * rendered using stroke paint with default stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testCubicPathFloatDashedStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d);
        // draw the path
        drawCubicPathFloat(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if cubic path could be clipped by an ellipse shape. Path is
     * rendered using stroke paint with default stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testCubicPathDoubleDashedStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d);
        // draw the path
        drawCubicPathDouble(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if quadratic path could be clipped by an ellipse shape. Path is
     * rendered using stroke paint with default stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testQuadraticPathFloatDashedStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d);
        // draw the path
        drawQuadraticPathFloat(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if quadratic path could be clipped by an ellipse shape. Path is
     * rendered using stroke paint with default stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testQuadraticPathDoubleDashedStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d);
        // draw the path
        drawQuadraticPathDouble(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if closed crossed path could be clipped by an ellipse shape. Path is
     * rendered using stroke paint with default stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testCrossedClosedPathFloatDashedStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d);
        // draw the path
        drawCrossedPathFloat(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if closed crossed path could be clipped by an ellipse shape. Path is
     * rendered using stroke paint with default stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testCrossedClosedPathDoubleDashedStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d);
        // draw the path
        drawCrossedPathDouble(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if closed path could be clipped by an ellipse shape. Path is
     * rendered using stroke paint with default stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClosedPathFloatDashedStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d);
        // draw the path
        drawClosedPathFloat(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if closed path could be clipped by an ellipse shape. Path is
     * rendered using stroke paint with default stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClosedPathDoubleDashedStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d);
        // draw the path
        drawClosedPathDouble(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if line path could be clipped by an ellipse shape. Path is
     * rendered using stroke paint with thick stroke width.
     * Line path is constructed using new Path.Float()
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testLinePathFloatDashedThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // set stroke width
        CommonRenderingStyles.setStrokeThickWidth(graphics2d);
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d);
        // draw the path
        drawLinePathFloat(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if line path could be clipped by an ellipse shape. Path is
     * rendered using stroke paint with thick stroke width.
     * Line path is constructed using new Path.Double()
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testLinePathDoubleDashedThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // set stroke width
        CommonRenderingStyles.setStrokeThickWidth(graphics2d);
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d);
        // draw the path
        drawLinePathDouble(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if cubic path could be clipped by an ellipse shape. Path is
     * rendered using stroke paint with thick stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testCubicPathFloatDashedThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // set stroke width
        CommonRenderingStyles.setStrokeThickWidth(graphics2d);
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d);
        // draw the path
        drawCubicPathFloat(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if cubic path could be clipped by an ellipse shape. Path is
     * rendered using stroke paint with thick stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testCubicPathDoubleDashedThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // set stroke width
        CommonRenderingStyles.setStrokeThickWidth(graphics2d);
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d);
        // draw the path
        drawCubicPathDouble(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if quadratic path could be clipped by an ellipse shape. Path is
     * rendered using stroke paint with thick stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testQuadraticPathFloatDashedThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // set stroke width
        CommonRenderingStyles.setStrokeThickWidth(graphics2d);
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d);
        // draw the path
        drawQuadraticPathFloat(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if quadratic path could be clipped by an ellipse shape. Path is
     * rendered using stroke paint with thick stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testQuadraticPathDoubleDashedThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // set stroke width
        CommonRenderingStyles.setStrokeThickWidth(graphics2d);
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d);
        // draw the path
        drawQuadraticPathDouble(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if closed crossed path could be clipped by an ellipse shape. Path is
     * rendered using stroke paint with thick stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testCrossedClosedPathFloatDashedThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // set stroke width
        CommonRenderingStyles.setStrokeThickWidth(graphics2d);
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d);
        // draw the path
        drawCrossedPathFloat(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if closed crossed path could be clipped by an ellipse shape. Path is
     * rendered using stroke paint with thick stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testCrossedClosedPathDoubleDashedThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // set stroke width
        CommonRenderingStyles.setStrokeThickWidth(graphics2d);
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d);
        // draw the path
        drawCrossedPathDouble(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if closed path could be clipped by an ellipse shape. Path is
     * rendered using stroke paint with thick stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClosedPathFloatDashedThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // set stroke width
        CommonRenderingStyles.setStrokeThickWidth(graphics2d);
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d);
        // draw the path
        drawClosedPathFloat(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if closed path could be clipped by an ellipse shape. Path is
     * rendered using stroke paint with thick stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClosedPathDoubleDashedThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // set stroke width
        CommonRenderingStyles.setStrokeThickWidth(graphics2d);
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d);
        // draw the path
        drawClosedPathDouble(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Entry point to the test suite.
     * 
     * @param args
     *            graphics test configuration
     */
    public static void main(String[] args)
    {
        new Paths().runTestSuite(args);
    }
}

